const path = require('path')
const fs = require('fs-extra')

const {
  NODE_MODULES_PATTERN,
  CHANGELOG_PATTERN,
  GITHUB_SKELETON_PATTERN,
  GITHUB_PATH,
  CODEBASE_SPECIFIC_WORKFLOWS_PATH
} = require('../constants')

const readPackageJson = folder => {
  const packageJsonFile = path.join(folder, 'package.json')

  const configData = fs.readFileSync(packageJsonFile)

  return JSON.parse(configData)
}

const writePackageJson = (folder, config) => {
  const packageJsonFile = path.join(folder, 'package.json')
  const modifiedConfigData = JSON.stringify(config, null, 2).concat('\n')

  fs.writeFileSync(packageJsonFile, modifiedConfigData)
}

const copyTemplatePackageJsonFile = async folder => {
  const packageJsonFile = path.join(folder, 'package.json')
  const packageSkeletonJsonFile = path.join(folder, 'package.json.skeleton')

  if (fs.existsSync(packageSkeletonJsonFile)) {
    await fs.remove(packageJsonFile)
    await fs.rename(packageSkeletonJsonFile, packageJsonFile)
  }
}

const copyTemplateGitIgnoreFile = async folder => {
  const gitIgnoreFile = path.join(folder, '.gitignore')
  const gitIgnoreSkeletonFile = path.join(folder, '.gitignore.skeleton')

  if (fs.existsSync(gitIgnoreSkeletonFile)) {
    await fs.rename(gitIgnoreSkeletonFile, gitIgnoreFile)
  }
}

const copyTemplateEslintrc = async folder => {
  const eslintrcFile = path.join(folder, '.eslintrc')
  const eslintrcSkeletonFile = path.join(folder, '.eslintrc.skeleton')

  if (fs.existsSync(eslintrcSkeletonFile)) {
    await fs.rename(eslintrcSkeletonFile, eslintrcFile)
  }
}

const copyNpmrc = async folder => {
  const npmrcFile = path.join(folder, '.npmrc')
  const npmrcFileSkeletonFile = path.join(folder, '.npmrc.skeleton')

  if (fs.existsSync(npmrcFileSkeletonFile)) {
    await fs.rename(npmrcFileSkeletonFile, npmrcFile)
  }
}

const copySPAWorkflows = async (sourceDir, destDir) => {
  const workflowsDir = path.join(
    sourceDir,
    GITHUB_PATH,
    CODEBASE_SPECIFIC_WORKFLOWS_PATH,
    'spa'
  )
  const targetDir = path.join(destDir, GITHUB_PATH, 'workflows')

  await fs.copy(workflowsDir, targetDir)
}

const copyAllFiles = async (sourceDir, destDir) => {
  const isSkeletonFile = src =>
    !src.match(NODE_MODULES_PATTERN) &&
    !src.match(CHANGELOG_PATTERN) &&
    !src.match(GITHUB_SKELETON_PATTERN)

  await fs.copy(sourceDir, destDir, { filter: isSkeletonFile })
}

module.exports = {
  readPackageJson,
  writePackageJson,
  copyTemplatePackageJsonFile,
  copyTemplateGitIgnoreFile,
  copyTemplateEslintrc,
  copyNpmrc,
  copySPAWorkflows,
  copyAllFiles,
}
